import re
from Device import Device
from conga_constants import FD_VAL_SUCCESS, FD_VAL_FAIL

FD_NEW_SUCCESS = 'New %s successfully added to cluster'
FD_UPDATE_SUCCESS = 'Fence device %s successfully updated'
FD_NEW_FAIL = 'No agent type %s in shared device list'

FI_NEW_FAIL = 'No instance type %s in fence instance list'

FD_PROVIDE_AGENT = 'A valid agent type must be provided for each Fence Device'
FD_PROVIDE_NAME = 'A unique name must be provided for each Fence Device'
FD_PROVIDE_PATH = 'An xCAT path must be provided for each xCAT Fence Device'
FD_PROVIDE_SERVER = 'A server address must be provided for this Fence Device'
FD_PROVIDE_CSERVER = 'A cserver address must be provided for this Egenera Fence Device'
FD_PROVIDE_IP = 'An IP address must be provided for this Fence Device'
FD_PROVIDE_HOSTNAME = 'A host name must be provided for this Fence Device'
FD_PROVIDE_LOGIN = 'A login name must be provided for this Fence Device'
FD_PROVIDE_PASSWD = 'A password must be provided for this Fence Device'
FD_PROVIDE_DEVICE = 'A device must be provided for this Fence Device'
FD_PROVIDE_PORT = 'A port must be provided for this Fence Device'
FD_PROVIDE_IPMILAN_AUTH = 'Authentication type must be "none" (or blank), "md5", or "password"'

FI_PROVIDE_PARENT = 'A Fence Device name must be provided for this Fence'
FI_PROVIDE_SWITCH = 'A switch address must be provided for this Fence'
FI_PROVIDE_PORT = 'A port value must be provided for this Fence'
FI_PROVIDE_BLADE = 'A Blade must be specified for this Fence'
FI_PROVIDE_DOMAIN = 'A Domain must be specified for this Fence'
FI_PROVIDE_IPADDRESS = 'An IP address must be provided for this Fence'
FI_PROVIDE_ELPAN = 'A LPAN value must be provided for this Egenera Fence'
FI_PROVIDE_EPSERVER = 'A PServer value must be provided for this Egenera Fence'
FI_PROVIDE_NODENAME = 'A Node Name value must be provided for this SCSI Fence'

ILLEGAL_CHARS_REPLACED = 'Illegal characters were replaced by underscores. Feel free to set a new value.'

# For every new fence added, there will be a fence instance and fence device
# form needed. There will also be a populate and a validate method for each
# form. The methods will need to be added to this file, and the four hash
# tables (self.fi_populate, self.fi_validate, self.fd_populate,
# self.fd_validate) must be updated. The methods will use fields in the
# forms to set_text and to check.

FENCE_OPTS = {
	'fence_apc':			'APC Power Device',
	'fence_wti':			'WTI Power Device',
	'fence_brocade':		'Brocade Switch',
	'fence_vixel':			'Vixel SAN Switch',
	'fence_gnbd':			'Global Network Block Device',
	'fence_sanbox2':		'QLogic SANBox2',
	'fence_bladecenter':	'IBM Blade Center',
	'fence_mcdata':			'McDATA SAN Switch',
	'fence_egenera':		'Egenera SAN Controller',
	'fence_bullpap':		'Bull PAP',
	'fence_xvm':			'Virtual Machine Fencing',
	'fence_scsi':			'SCSI Reservation',
	'fence_ilo':			'HP ILO Device',
	'fence_ipmilan':		'IPMI Lan',
	'fence_drac':			'Dell DRAC',
	'fence_rsa':			'IBM RSA II Device',
	'fence_rps10':			'RPS10 Serial Switch',
	'fence_manual':			'Manual Fencing'
}

FENCE_SHARED = {
	'fence_apc':			True,
	'fence_wti':			True,
	'fence_brocade':		True,
	'fence_vixel':			True,
	'fence_gnbd':			True,
	'fence_sanbox2':		True,
	'fence_bladecenter':	True,
	'fence_mcdata':			True,
	'fence_egenera':		True,
	'fence_bullpap':		True,
	'fence_xvm':			True,
	'fence_scsi':			True,
	'fence_ilo':			False,
	'fence_ipmilan':		False,
	'fence_drac':			False,
	'fence_rsa':			False,
	'fence_rps10':			False,
	'fence_manual':			False
}

PRETTY_NAME_ATTRS = {
	'port':					'Port',
	'blade':				'Blade',
	'switch':				'Switch',
	'ipaddr':				'IP Address',
	'ipaddress':			'IP Address',
	'nodename':				'Node Name',
	'lpan':					'LPAN',
	'lanplus':				'Lanplus',
	'pserver':				'PServer',
	'login':				'Login',
	'passwd':				'Password',
	'name':					'Name',
	'server':				'Server',
	'domain':				'Domain',
	'hostname':				'Hostname',
	'path':					'Path',
	'cserver':				'CServer'
}

FENCE_FI_ATTRS = {
	'fence_apc':			['port', 'switch'],
	'fence_wti':			['port'],
	'fence_brocade':		['port'],
	'fence_vixel':			['port'],
	'fence_gnbd':			['ipaddress'],
	'fence_sanbox2':		['port'],
	'fence_bladecenter':	['blade'],
	'fence_mcdata':			['port'],
	'fence_egenera':		['lpan', 'pserver'],
	'fence_bullpap':		['domain'],
	'fence_xvm':			['domain'],
	'fence_scsi':			['nodename'],
	'fence_ilo':			[],
	'fence_ipmilan':		[],
	'fence_drac':			[],
	'fence_rsa':			[],
	'fence_rps10':			[],
	'fence_manual':			[]
}

FENCE_FD_ATTRS = {
	'fence_apc': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_wti': ['name', 'ipaddr', 'passwd'],
	'fence_brocade': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_vixel': ['name', 'ipaddr', 'passwd'],
	'fence_gnbd': ['name', 'servers'],
	'fence_sanbox2': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_bladecenter': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_mcdata': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_egenera': ['name', 'cserver'],
	'fence_bullpap': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_xvm': ['name'],
	'fence_scsi': ['name'],
	'fence_ilo': ['name', 'hostname', 'login', 'passwd'],
	'fence_ipmilan': ['name', 'ipaddr', 'login', 'passwd', 'lanplus', 'auth'],
	'fence_drac': ['name', 'ipaddr', 'login', 'passwd'],
	'fence_rsa': ['name', 'hostname', 'login', 'passwd'],
	'fence_rps10': ['name', 'device', 'port'],
	'fence_manual': ['name']
}

ILLEGAL_CHARS = re.compile(':| ')

def makeNCName(name):
	### name must conform to relaxNG ID type ##
	return ILLEGAL_CHARS.sub('_', name)

def check_unique_fd_name(model, name):
	fds = model.getFenceDevices()
	for fd in fds:
		if fd.getName() == name:
			return False
	return True

def validateNewFenceDevice(form, model):
	from FenceDevice import FenceDevice
	fencedev = FenceDevice()

	try:
		ret = validate_fencedevice(form, model, fencedev)
		if len(ret) < 1:
			fencedevptr = model.getFenceDevicePtr()
			fencedevptr.addChild(fencedev)
			model.setModified(True)
			return (FD_VAL_SUCCESS, fencedev.getAttribute('name'))
	except Exception, e:
		ret = [ FD_PROVIDE_AGENT ]

	return (FD_VAL_FAIL, ret)

def validateFenceDevice(form, model):
	from FenceDevice import FenceDevice
	try:
		old_fence_name = form['orig_name'].strip()
		if not old_fence_name:
			raise Exception, 'blank'
	except Exception, e:
		return (FD_VAL_FAIL, [ FD_PROVIDE_NAME ])

	fencedev = None
	try:
		fencedevs = model.getFenceDevices()
		for fd in fencedevs:
			if fd.getName().strip() == old_fence_name:
				fencedev = fd
				break
		if fencedev is None:
			raise Exception, 'fencedev is None'
	except Exception, e:
		return (FD_VAL_FAIL, [ FD_PROVIDE_NAME ])

	try:
		ret = validate_fencedevice(form, model, fencedev, fence_edit=True)
		if len(ret) < 1:
			model.setModified(True)
			return (FD_VAL_SUCCESS, fencedev.getAttribute('name'))
	except Exception, e:
		ret = [ FD_PROVIDE_NAME ]

	return (FD_VAL_FAIL, ret)

def val_apc_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_wti_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	return errors

def val_brocade_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_vixel_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	return errors

def val_mcdata_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_gnbd_fd(form, fencedev):
	errors = list()

	try:
		server = form['servers'].strip()
		if not server:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_SERVER)

	fencedev.addAttribute('servers', server)
	return errors

def val_egenera_fd(form, fencedev):
	errors = list()

	try:
		cserver = form['cserver'].strip()
		if not cserver:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_CSERVER)

	fencedev.addAttribute('cserver', cserver)
	return errors

def val_sanbox2_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_bladecenter_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_bullpap_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_noop_fd(dummy, _dummy):
	return []

# non-shared devices

def val_rsa_fd(form, fencedev):
	errors = list()

	try:
		hostname = form['hostname'].strip()
		if not hostname:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_HOSTNAME)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('hostname', hostname)
	fencedev.addAttribute('login', log)
	return errors

def val_drac_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_rps10_fd(form, fencedev):
	errors = list()

	try:
		device = form['device'].strip()
		if not device:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_DEVICE)

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_PORT)

	fencedev.addAttribute('device', device)
	fencedev.addAttribute('port', port)
	return errors

def val_ipmilan_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	try:
		auth_type = form['auth_type'].strip().lower()
		if not auth_type or auth_type == 'none':
			fencedev.removeAttribute('auth_type')
		elif auth_type == 'password' or auth_type == 'md5':
			fencedev.setAttribute('auth_type', auth_type)
		else:
			errors.append(FD_PROVIDE_IPMILAN_AUTH)
	except KeyError, e:
		fencedev.removeAttribute('auth_type')
	except Exception, e:
		errors.append(FD_PROVIDE_IPMILAN_AUTH)

	try:
		lanplus = form['lanplus'].strip().lower()
		if not lanplus or lanplus == '0' or lanplus == 'false' or lanplus == 'off':
			fencedev.removeAttribute('lanplus')
		else:
			fencedev.setAttribute('lanplus', '1')
	except Exception, e:
		fencedev.removeAttribute('lanplus')

	fencedev.addAttribute('ipaddr', ip)
	fencedev.addAttribute('login', log)
	return errors

def val_ilo_fd(form, fencedev):
	errors = list()

	try:
		hostname = form['hostname'].strip()
		if not hostname:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_HOSTNAME)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	fencedev.addAttribute('hostname', hostname)
	fencedev.addAttribute('login', log)
	return errors

FD_VALIDATE = {
	'fence_apc':			val_apc_fd,
	'fence_wti':			val_wti_fd,
	'fence_brocade':		val_brocade_fd,
	'fence_vixel':			val_vixel_fd,
	'fence_gnbd':			val_gnbd_fd,
	'fence_sanbox2':		val_sanbox2_fd,
	'fence_bladecenter':	val_bladecenter_fd,
	'fence_mcdata':			val_mcdata_fd,
	'fence_egenera':		val_egenera_fd,
	'fence_bullpap':		val_bullpap_fd,
	'fence_xvm':			val_noop_fd,
	'fence_scsi':			val_noop_fd,
	'fence_ilo':			val_ilo_fd,
	'fence_ipmilan':		val_ipmilan_fd,
	'fence_drac':			val_drac_fd,
	'fence_rsa':			val_rsa_fd,
	'fence_rps10':			val_rps10_fd,
	'fence_manual':			val_noop_fd
}

def validate_fencedevice(form, model, fencedev, fence_edit=False):
	try:
		fence_name = form['name'].strip()
		if not fence_name:
			raise Exception, 'blank'
		fence_name = makeNCName(fence_name)
	except Exception, e:
		return [ FD_PROVIDE_NAME ]

	name_change = False
	if fence_edit is True:
		try:
			old_fence_name = form['orig_name'].strip()
			if not old_fence_name:
				raise Exception, 'blank'
		except Exception, e:
			return [ FD_PROVIDE_NAME ]
		if old_fence_name != fence_name:
			if check_unique_fd_name(model, fence_name) is False:
				return [ FD_PROVIDE_NAME ]
			name_change = True
	else:
		if check_unique_fd_name(model, fence_name) is False:
			return [ FD_PROVIDE_NAME ]

	try:
		fence_agent = form['fence_type'].strip()
		if not fence_agent:
			raise Exception, 'blank agent'
	except Exception, e:
		return [ FD_PROVIDE_AGENT ]

	fencedev.addAttribute('name', fence_name)
	fencedev.addAttribute('agent', fence_agent)

	try:
		ret = FD_VALIDATE[fence_agent](form, fencedev)
		if len(ret) < 1 and name_change is True:
			try:
				model.rectifyNewFencedevicenameWithFences(old_fence_name, fence_name)
			except:
				return [ FD_NEW_FAIL % fence_agent ]
		return ret
	except:
		return [ FD_NEW_FAIL % fence_agent ]

# Validation Methods for Fence Instances

def val_apc_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	try:
		switch = form['switch'].strip()
		if not switch:
			raise KeyError, 'blank'
		fenceinst.addAttribute('switch', switch)
	except KeyError, e:
		# switch is optional
		fenceinst.removeAttribute('switch')
	except Exception, e:
		errors.append(FI_PROVIDE_SWITCH)

	fenceinst.addAttribute('port', port)
	return errors

def val_wti_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	fenceinst.addAttribute('port', port)
	return errors

def val_brocade_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	fenceinst.addAttribute('port', port)
	return errors

def val_vixel_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	fenceinst.addAttribute('port', port)
	return errors

def val_gnbd_fi(form, fenceinst):
	errors = list()

	try:
		ip = form['ipaddress'].strip()
		if not ip:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_IPADDRESS)

	fenceinst.addAttribute('ipaddress', ip)
	return errors

def val_sanbox2_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	fenceinst.addAttribute('port', port)
	return errors

def val_bladecenter_fi(form, fenceinst):
	errors = list()

	try:
		blade = form['blade'].strip()
		if not blade:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_BLADE)

	fenceinst.addAttribute('blade', blade)
	return errors

def val_mcdata_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	fenceinst.addAttribute('port', port)
	return errors

def val_egenera_fi(form, fenceinst):
	errors = list()

	try:
		lpan = form['lpan'].strip()
		if not lpan:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_ELPAN)

	try:
		pserver = form['pserver'].strip()
		if not pserver:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_ELPAN)

	fenceinst.addAttribute('lpan', lpan)
	fenceinst.addAttribute('pserver', pserver)
	return errors

def val_bullpap_fi(form, fenceinst):
	errors = list()

	try:
		domain = form['domain'].strip()
		if not domain:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_DOMAIN)

	fenceinst.addAttribute('domain', domain)
	return errors

def val_xvm_fi(form, fenceinst):
	errors = list()

	try:
		domain = form['domain'].strip()
		if not domain:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_DOMAIN)

	fenceinst.addAttribute('domain', domain)
	return errors

def val_scsi_fi(form, fenceinst):
	errors = list()

	try:
		nodename = form['nodename'].strip()
		if not nodename:
			raise Exception, 'blank'
	except Exception, e:
		errors.append(FI_PROVIDE_NODENAME)

	fenceinst.addAttribute('nodename', nodename)
	return errors

def val_noop_fi(dummy, _dummy):
	return []

FI_VALIDATE = {
	'fence_apc':			val_apc_fi,
	'fence_wti':			val_wti_fi,
	'fence_brocade':		val_brocade_fi,
	'fence_vixel':			val_vixel_fi,
	'fence_gnbd':			val_gnbd_fi,
	'fence_sanbox2':		val_sanbox2_fi,
	'fence_bladecenter':	val_bladecenter_fi,
	'fence_mcdata':			val_mcdata_fi,
	'fence_egenera':		val_egenera_fi,
	'fence_bullpap':		val_bullpap_fi,
	'fence_xvm':			val_xvm_fi,
	'fence_scsi':			val_scsi_fi,
	'fence_ilo':			val_noop_fi,
	'fence_ipmilan':		val_noop_fi,
	'fence_drac':			val_noop_fi,
	'fence_rsa':			val_noop_fi,
	'fence_rps10':			val_noop_fi,
	'fence_manual':			val_noop_fi
}

def validate_fenceinstance(form, parent_name, fence_agent):
	try:
		if not parent_name.strip():
			return (FD_VAL_FAIL, [ FI_PROVIDE_PARENT ])
	except:
		return (FD_VAL_FAIL, [ FI_PROVIDE_PARENT ])

	fenceinst = Device()
	fenceinst.addAttribute('name', parent_name)

	try:
		ret = FI_VALIDATE[fence_agent](form, fenceinst)
		if len(ret) > 0:
			return (FD_VAL_FAIL, ret)
	except Exception, e:
		return (FD_VAL_FAIL, [ FI_NEW_FAIL % fence_agent ])

	return (FD_VAL_SUCCESS, fenceinst)
