from conga_constants import LUCI_DEBUG_MODE, LUCI_DEBUG_VERBOSITY
from syslog import openlog, closelog, syslog, \
		LOG_DAEMON, LOG_PID, LOG_NDELAY, LOG_INFO, \
		LOG_WARNING, LOG_AUTH, LOG_DEBUG

# Exception class for the LuciSyslog facility
class LuciSyslogError(Exception):
	def __init__(self, msg):
		Exception.__init__(self, msg)

# Facility that provides centralized syslog(3) functionality for luci
class LuciSyslog:
	def __init__(self):
		self.__init = 0
		try:
			openlog('luci', LOG_PID | LOG_NDELAY, LOG_DAEMON)
			self.__init = 1
		except:
			raise LuciSyslogError, 'unable to setup syslog functionality.'

	def info(self, msg):
		if not self.__init:
			return
		try:
			syslog(LOG_INFO, msg)
		except:
			pass
			#raise LuciSyslogError, 'syslog info call failed'

	def warn(self, msg):
		if not self.__init:
			return
		try:
			syslog(LOG_WARNING, msg)
		except:
			pass
			#raise LuciSyslogError, 'syslog warn call failed'

	def private(self, msg):
		if not self.__init:
			return
		try:
			syslog(LOG_AUTH, msg)
		except:
			pass
			#raise LuciSyslogError, 'syslog private call failed'

	def debug_verbose(self, msg):
		if not LUCI_DEBUG_MODE or LUCI_DEBUG_VERBOSITY < 2 or not self.__init:
			return

		msg_len = len(msg)
		if msg_len < 1:
			return

		while True:
			cur_len = min(msg_len, 800)
			cur_msg = msg[:cur_len]
			try:
				syslog(LOG_DEBUG, cur_msg)
			except:
				pass

			msg_len -= cur_len
			if msg_len > 0:
				msg = msg[cur_len:]
			else:
				break

	def debug(self, msg):
		if not LUCI_DEBUG_MODE or not self.__init:
			return
		try:
			syslog(LOG_DEBUG, msg)
		except:
			pass
			#raise LuciSyslogError, 'syslog debug call failed'

	def close(self):
		try:
			self.__init = 0
			closelog()
		except:
			pass
